@@ -15,6 +15,9 @@ def get_test_scenarios():
     with open("tests/expected_results.json", "r") as f:
         expected_results = json.load(f)
 
+    # set local timezone
+    iglu.utils.set_local_tz(expected_results["config"]["local_tz"])
+
     # Filter scenarios for ROC method
     return [
         scenario
